import { Component, ChangeDetectorRef, ApplicationRef, AfterViewChecked } from '@angular/core';
import { MdDialog } from '@angular/material';

import { ParamService } from './services/param/param.service';
import { HttpService } from './services/http/http.service';
import { InternationalisationService, Language, LanguageCode } from './services/internationalisation/internationalisation.service';
import { Observer } from './services/observer';
import { ErrorService } from './services/error/error.service';
import { AlertDialog } from './components/alert-dialog/alert-dialog.component';
import { FormulaireService } from './services/formulaire/formulaire.service';

@Component({
  selector: 'nghyd-app',
  template: `
  <select [(ngModel)]=_currentLanguage (change)=onSelectLang($event)>
    <option *ngFor="let l of intlService.languages" [value]=l.code>{{l.label}}</option>
  </select>

  <courbe-remous></courbe-remous>
  <!--
  <regime-uniforme></regime-uniforme>
  <section-param></section-param>
  <lechapt-calmon></lechapt-calmon>
  <cond-distri></cond-distri>
  <select name="Language" (change)=selectLang($event)>
    <option *ngFor="let l of intlService.languages" [value]=l.code>{{l.label}}</option>
  </select>
  <h1>{{title}}</h1>
  <param-input symbol="Q"></param-input>
  <param-input symbol="Q"></param-input>
  <param-input symbol="Ks"></param-input>
  -->
  `,
  providers: [ParamService, InternationalisationService, HttpService, FormulaireService]
})
export class AppComponent implements Observer {
  private _currentLanguage: LanguageCode;

  private _displayErrorDialog: boolean = false;

  constructor(private intlService: InternationalisationService, private appRef: ApplicationRef, private dialog: MdDialog, private errorService: ErrorService) { }

  private initLocale() {
    let docLocale: string = document['locale'] as string;
    this.intlService.setLocale(docLocale);
    this._currentLanguage = this.intlService.currentLanguage.code;
  }

  ngOnInit() {
    this.initLocale();
    this.subscribeErrorService();
  }

  /**
   * abonnement au service d'erreurs
   */
  private subscribeErrorService() {
    this.errorService.addObserver(this);
  }

  /*
  angular i18n ne prend pas en charge le changement dynamique de langue :
  https://github.com/angular/angular/issues/13905
  */

  private onSelectLang(evt: any) {
    let v = evt.target.value;
    let lc = LanguageCode[v];
    this.intlService.setLocale(lc);

    this._currentLanguage = this.intlService.currentLanguage.code;

    let tag = this.intlService.currentLanguage.tag;
    document['locale'] = tag;

    // location.reload(true);
    // this.cdRef.markForCheck();
    // this.cdRef.detectChanges();
    this.appRef.tick();
  }

  // interface Observer

  update(data: any): void {
    // on ouvre un dialogue avec le message d'erreur reçu
    if (this._displayErrorDialog) {
      let dialogRef = this.dialog.open(AlertDialog);
      let ad: AlertDialog = dialogRef.componentInstance;
      ad.text = String(data);
    }
    else
      console.log(data);
  }
}
