import { Component } from '@angular/core';

import { NgParameter } from '../../calculators/generic/ngparam';

@Component({
    selector: 'calc-results',
    templateUrl: './calculator-results.component.html',
    styles: [`
    .result_label {
        text-align: right;
        padding-top:10px;
        padding-bottom:10px;
        padding-right:10px;   
    }
    .result_value {
        text-align: center;
        padding-left:30px;   
        padding-right:30px;   
    }
    .result_id_0 {
        background-color: #f0f0f0;
    }
    .result_id_2 {
        font-weight: bold;
    }
    `
    ]
})
export class CalculatorResultsComponent {
    /**
     * résultats fixed (true) ou variés (false)
     */
    private _isFixed: boolean;

    private _fixedResults: Object[] = [];

    /**
     * affichage de la table des résultats variés
     */
    private get showVarResults(): boolean {
        return this._varResults.length > 0;
    }

    private _variableParamHeader: string;

    private _variableResultHeader: string;

    /**
     * tableau de valeurs de la table des résultats variés
     */
    private _varResults: Object[] = [];

    /**
     * tableau de valeurs du graphe des résultats variés
     */
    private _varGraph: number[] = [];

    /*
    * config du graphe
    */
    private graph_type = 'line';
    private graph_data = {};
    private graph_options = {
        responsive: true,
        maintainAspectRatio: false,
        animation: {
            duration: 0
        },
        legend: {
            display: false
        },
        title: {
            display: true,
            text: ""
        }
    };

    public addFixedResult(p: NgParameter, v: number, fixedPrec: number, displaySymbol: boolean) {
        this._fixedResults.push({ "label": this.paramLabel(p, displaySymbol), "value": v.toFixed(fixedPrec) });
    }

    public addVarResult(paramVal: number, resVal: number, fixedPrec: number) {
        this._varResults.push({ "param": paramVal.toFixed(fixedPrec), "result": resVal.toFixed(fixedPrec) });
        this._varGraph.push(resVal);
    }

    public generateGraph() {
        let labs = [];
        let dat = [];
        for (let i in this._varResults) {
            let r = this._varResults[i];
            labs.push(r["param"]);
            dat.push(this._varGraph[i]);
        }

        this.graph_data = {
            labels: labs,
            datasets: [
                {
                    label: "",
                    data: dat
                }
            ]
        };
    }

    public reset(fixed: boolean) {
        this._isFixed = fixed;
        this._fixedResults = [];
        this._varResults = [];
        this._varGraph = [];
        this._variableParamHeader = undefined;
        this._variableResultHeader = undefined;
        this.graph_options.title.text = undefined;
    }

    public setVariableParamHeader(h: string) {
        this._variableParamHeader = h;
    }

    public setVariableParamHeaderFromParameter(p: NgParameter, displaySymbol: boolean) {
        this._variableParamHeader = this.paramLabel(p, displaySymbol);
    }

    public setVariableResultHeader(h: string) {
        this._variableResultHeader = h;
    }

    public setVariableResultHeaderFromParameter(p: NgParameter) {
        this._variableResultHeader = this.paramLabel(p, true);
    }

    public setGraphTitle(t: string) {
        this.graph_options.title.text = t;
    }

    private paramLabel(p: NgParameter, displaySymbol: boolean): string {
        let res = "";
        if (displaySymbol)
            res += p.symbol;
        if (p.label != undefined && p.label != "") {
            if (res.length > 0)
                res += ":";
            res += p.label;
        }
        if (p.unit != undefined && p.unit != "")
            res += " (" + p.unit + ")";
        return res;
    }

    private getFixedResultClass(i: number) {
        if (this._isFixed && i == this._fixedResults.length - 1)
            return "result_id_2"
        return "result_id_" + String(i & 1);
    }

    private getVarResultClass(i: number) {
        return "result_id_" + String(i & 1);
    }
}
