import { Component, Input, ViewChild, ElementRef, AfterViewInit } from '@angular/core';

@Component({
    selector: 'calc-canvas',
    template: `<canvas #canvas
    [attr.width] = '_width'
    [attr.height] = '_height'>
    </canvas>
    `
})
export class CalcCanvasComponent implements AfterViewInit {
    private _width: number = 300;
    private _height: number = 200;

    public get width(): number {
        return this._calcCanvas.nativeElement.width;
    }

    @Input()
    public set width(w: number) {
        this._width = w;
    }

    public get height(): number {
        return this._calcCanvas.nativeElement.height;
    }

    @Input()
    public set height(h: number) {
        this._height = h;
    }

    private _context2d: CanvasRenderingContext2D;

    @ViewChild("canvas")
    private _calcCanvas: ElementRef;

    ngAfterViewInit() { // wait for the view to init before using the element
        this._context2d = this._calcCanvas.nativeElement.getContext("2d");

        //this._context2d.fillStyle = 'blue';
        // this._context2d.fillRect(10, 10, 350, 350);

        // this.drawRect(0, 0, this._width, this._height);
    }

    public clear() {
        this._context2d.clearRect(0, 0, this.width, this.height);
    }

    public setStrokeColor(r: number, g: number, b: number) {
        let col: string = "rgb(" + r + "," + g + "," + b + ")";
        this._context2d.strokeStyle = col;
    }

    public setFillColor(r: number, g: number, b: number) {
        let col: string = "rgb(" + r + "," + g + "," + b + ")";
        this._context2d.fillStyle = col;
    }

    public setFont(f: string) {
        this._context2d.font = f;
    }

    public fillText(s: string, x: number, y: number, align: string = undefined) {
        if (align != undefined)
            this._context2d.textAlign = align;
        this._context2d.fillText(s, x, y);
    }

    public setLineWidth(w: number) {
        this._context2d.lineWidth = w;
    }

    public setLineDash(d: number[]) {
        this._context2d.setLineDash(d);
    }

    public resetLineDash() {
        this._context2d.setLineDash([]);
    }

    public drawRect(x1: number, y1: number, w: number, h: number) {
        this._context2d.strokeRect(x1, y1, w, h);
    }

    public drawLine(x1: number, y1: number, x2: number, y2: number) {
        this._context2d.beginPath();
        this._context2d.moveTo(x1, y1);
        this._context2d.lineTo(x2, y2);
        this._context2d.stroke();
    }

    /**
     * 
     * @param x The x axis of the coordinate for the ellipse's center.
     * @param y The y axis of the coordinate for the ellipse's center.
     * @param radiusX The ellipse's major-axis radius.
     * @param radiusY The ellipse's minor-axis radius.
     * @param rotation The rotation for this ellipse, expressed in radians
     * @param startAngle The starting point, measured from the x axis, from which it will be drawn, expressed in radians
     * @param endAngle The end ellipse's angle to which it will be drawn, expressed in radians
     */
    public drawEllipse(x: number, y: number, radiusX: number, radiusY: number, rotation: number, startAngle: number, endAngle: number) {
        this._context2d.beginPath();
        this._context2d.ellipse(x, y, radiusX, radiusY, rotation, startAngle, endAngle);
        this._context2d.stroke();
    }

    public get context2d(): CanvasRenderingContext2D {
        return this._context2d;
    }
}
