import { Component, Input, Output, EventEmitter } from '@angular/core';

import { ParamDefinition } from 'jalhyd';

import { NgParameter, ParamRadioConfig } from '../../calculators/generic/ngparam';
import { FieldSet } from '../../calculators/generic/formulaire';

@Component({
    selector: "field-set",
    templateUrl: "./field-set.html",
    styles: [`.fieldset_title {
        font-weight: bold;
        margin: 0.3em 0 0.2em;
        padding: 0.3em;
        background-color: #eeeeee;
        }`,
        `.radio_param_header {
        width: 10em;
        }`
    ]
})
export class FieldSetComponent {
    /**
    * field set attribute
    */
    private _fieldSet: FieldSet;

    @Input()
    private set fieldSet(fs: FieldSet) {
        this._fieldSet = fs;
    }

    constructor() {
    }

    private hasRadioFix(): boolean {
        if (this._fieldSet.hasInputs)
            switch (this._fieldSet.getInput(0).radioConfig) {
                case ParamRadioConfig.FIX:
                    return false;

                default:
                    return true;
            }
        return false;
    }

    private hasRadioVar(): boolean {
        if (this._fieldSet.hasInputs)
            switch (this._fieldSet.getInput(0).radioConfig) {
                case ParamRadioConfig.VAR:
                case ParamRadioConfig.CAL:
                    return true;

                default:
                    return false;
            }
        return false;
    }

    private hasRadioCal(): boolean {
        if (this._fieldSet.hasInputs)
            switch (this._fieldSet.getInput(0).radioConfig) {
                case ParamRadioConfig.CAL:
                    return true;

                default:
                    return false;
            }
        return false;
    }

    /*
   * gestion des événements clic sur les radios :
   * réception d'un message du composant enfant (param-field)
   * cf. https://angular.io/guide/component-interaction#parent-listens-for-child-event
   */
    private onRadioClick(info: string) {
        // console.log("FieldSetComponent " + info);
        // on renvoie l'info au parent
        this.onRadio.emit(info);
    }

    @Output()
    private onRadio = new EventEmitter<string>();

    /**
     * réception d'un événement d'un select
     */
    private onSelectChanged(val: string) {
        //console.log("fieldset " + val);
        this.onSelectChange.emit(val); // on transmet au parent
    }

    @Output()
    private onSelectChange = new EventEmitter<string>();
}
