import { Component } from '@angular/core';

@Component({
    selector: 'remous-results',
    templateUrl: './remous-results.component.html',
    styles: [`
    .result_label {
        text-align: right;
        padding-top:10px;
        padding-bottom:10px;
        padding-right:10px;   
    }
    .result_value {
        text-align: center;
        padding-left:30px;   
        padding-right:30px;   
    }
    .result_id_0 {
        background-color: #f0f0f0;
    }
    .result_id_2 {
        font-weight: bold;
    }
    `
    ]
})
export class RemousResultsComponent {
    // /**
    //  * tirant imposé à l'amont
    //  */
    // private _Yamont: number;

    // /**
    //  * tirant imposé à l'aval
    //  */
    // private _Yaval: number;

    /**
     * pente du fond
     */
    private _penteFond: number;

    /**
     * hauteur de berge
     */
    private _hautBerge: number;

    /**
     * hauteur normale
     */
    private _hautNormale: number;

    /**
     * hauteur critique
     */
    private _hautCritique: number;

    /**
     * longueur du bief
     */
    private _longBief: number;

    /**
    * journal
    */
    private _logEntries: Object[] = [];

    /**
    * résultats numériques
    */
    private _results: Object[] = [];

    /**
    * tableau de valeurs du graphe des tirants
    */
    private _fluvialGraph: any = {};
    private _torrentGraph: any = {};

    /*
    * config du graphe
    */
    private graph_type = 'line';
    private graph_data = {};
    private graph_options = {
        responsive: true,
        maintainAspectRatio: true,
        animation: {
            duration: 0
        },
        legend: {
            display: false
        },
        title: {
            display: false,
            text: ""
        }
    };

    public reset() {
        this._results = [];
        this._logEntries = [];
        this._fluvialGraph = {};
        this._torrentGraph = {};
    }

    /**
     * transforme une cote en tenant compte du fond
     * @param x abscisse où se trouve la cote à transformer
     * @param y cote à transformer
     */
    private mapY(x: number, y: number) {
        let d: number;
        if (this._penteFond >= 0) {
            d = this._penteFond * (this._longBief - x);
        } else {
            d = -this._penteFond * x;
        }
        return y + d;
    }

    private drawLine(y0: number, ymax: number, color: string, fillColor: string = undefined) {
        let ys = [];
        ys.push({ x: 0, y: this.mapY(0, y0) });
        ys.push({ x: this._longBief, y: this.mapY(this._longBief, ymax) });
        // return { data: yFond, fill: true, tension: 0, borderColor: "#753F00", backgroundColor: "#753F00", pointRadius: 0 };
        return { data: ys, fill: fillColor != undefined, tension: 0, borderColor: color, backgroundColor: fillColor, pointRadius: 0 };
    }

    public generateGraph() {
        // http://www.chartjs.org/docs/latest/charts/line.html

        let ds: any = [];

        // abscisses

        let labs = [];
        for (let r of this._results)
            labs.push(+r["abs"]);

        // ligne de fond
        ds.push(this.drawLine(0, 0, "#753F00", "#753F00"));

        // ligne de berge
        ds.push(this.drawLine(this._hautBerge, this._hautBerge, "#C58F50"));

        // hauteur normale
        ds.push(this.drawLine(this._hautNormale, this._hautNormale, "#A4C537"));

        // hauteur critique
        ds.push(this.drawLine(this._hautCritique, this._hautCritique, "#FF0000"));

        // lignes d'eau torrentielle et fluviale

        let dataFlu = [];
        let dataTor = [];
        for (let r of this._results) {
            let x: number = +r["abs"];
            let yFlu: string = r["flu"];
            let yTor: string = r["tor"];
            if (yFlu != undefined && yFlu != "")
                dataFlu.push(this.mapY(x, +yFlu));
            if (yTor != undefined && yTor != "")
                dataTor.push(this.mapY(x, +yTor));
        }
        ds.push({ data: dataTor, tension: 0, borderColor: "#77A3CD", pointRadius: 5, backgroundColor: "#D1D0D4" });
        ds.push({ data: dataFlu, tension: 0, borderColor: "#0093BD", pointRadius: 5, backgroundColor: "#D1D0D4" });

        this.graph_data = {
            labels: labs,
            datasets: ds
        };
    }

    public addResult(x: string, f: string, t: string) {
        this._results.push({ "abs": x, "flu": f, "tor": t });
    }

    public addLogEntry(s: string) {
        this._logEntries.push(s);
    }

    // public setYamont(v: number) {
    //     this._Yamont = v;
    // }

    // public setYaval(v: number) {
    //     this._Yaval = v;
    // }

    public setPenteFond(v: number) {
        this._penteFond = v;
    }

    public setHauteurBerge(v: number) {
        this._hautBerge = v;
    }

    public setHauteurNormale(v: number) {
        this._hautNormale = v;
    }

    public setHauteurCritique(v: number) {
        this._hautCritique = v;
    }

    public setLongBief(v: number) {
        this._longBief = v;
    }

    public addXYFluvial(x: number, y: number) {
        this._fluvialGraph.push({ x: x, y: y });
    }

    public addXYTorrent(x: number, y: number) {
        this._torrentGraph.push({ x: x, y: y });
    }

    private getResultClass(i: number) {
        return "result_id_" + String(i & 1);
    }
}
