import { Component, ViewChild } from '@angular/core';

import { acSection } from 'jalhyd';

import { SectionCanvasComponent } from '../section-canvas/section-canvas.component';

@Component({
    selector: 'section-results',
    templateUrl: './section-results.component.html',
    styles: [`
    .result_label {
        text-align: right;
        padding-top:10px;
        padding-bottom:10px;
        padding-right:10px;   
    }
    .result_value {
        text-align: center;
        padding-left:30px;   
        padding-right:30px;   
    }
    .result_id_0 {
        background-color: #f0f0f0;
    }
    .result_id_2 {
        font-weight: bold;
    }
    `
    ]
})
export class SectionResultsComponent {
    /**
    * résultats
    */
    private _results: Object[] = [];

    private static labelColors: { [key: string]: any; } = {
        "Hs": { r: 255, g: 0, b: 0 },
        "Hsc": { r: 0, g: 0, b: 255 },
        "Yn": { r: 16, g: 128, b: 16 },
        "Yf": { r: 128, g: 128, b: 128 },
        "Yt": { r: 0, g: 0, b: 0 },
        "Yc": { r: 255, g: 128, b: 0 },
        "Yco": { r: 255, g: 0, b: 255 },
    };

    @ViewChild("sectionCanvas")
    private _sectionCanvas: SectionCanvasComponent;

    public reset() {
        this._results = [];
        this._sectionCanvas.reset();
    }

    public set section(section: acSection) {
        this._sectionCanvas.section = section;
    }

    public addResult(v: number, l: string, fixedPrec: number, drawLabel: string) {
        this._results.push({ "label": l, "value": v.toFixed(fixedPrec) });
        if (drawLabel != undefined) {
            this._sectionCanvas.addLevel(v, drawLabel + " = " + v.toFixed(fixedPrec), SectionResultsComponent.labelColors[drawLabel]);
        }
    }

    private getResultClass(i: number) {
        return "result_id_" + String(i & 1);
    }
}