import {
    ParamDomain, ComputeNodeType, ComputeNodeParameters, ParamsEquation, ParamsSectionRectang, ParamDefinition,
    ParamDomainValue, ParamCalculability
} from 'jalhyd';

import { NgParameter } from "../../calculators/generic/ngparam";
import { logObject } from '../../util';

export class ParamService {
    // private _params: ParamDefinition[];
    private _params: NgParameter[];

    constructor() {
        this._params = [];

        this.addParameters(ComputeNodeType.CondDistri);
        this.addParameters(ComputeNodeType.LechaptCalmon);
        this.addParameters(ComputeNodeType.SectionTrapeze);
        this.addParameters(ComputeNodeType.SectionRectangle);
        this.addParameters(ComputeNodeType.SectionCercle);
        this.addParameters(ComputeNodeType.SectionPuissance);
        this.addParameters(ComputeNodeType.RegimeUniformeTrapeze);
        this.addParameters(ComputeNodeType.RegimeUniformeRectangle);
        this.addParameters(ComputeNodeType.RegimeUniformeCercle);
        this.addParameters(ComputeNodeType.RegimeUniformePuissance);
        this.addParameters(ComputeNodeType.CourbeRemousCercle);
        this.addParameters(ComputeNodeType.CourbeRemousPuissance);
        this.addParameters(ComputeNodeType.CourbeRemousRectangle);
        this.addParameters(ComputeNodeType.CourbeRemousTrapeze);

        // précision de calcul

        let d = new ParamDomain(ParamDomainValue.INTERVAL, 1e-10, 100);
        let p = new ParamDefinition(ComputeNodeType.LechaptCalmon, 'Pr', d);
        p.calculability = ParamCalculability.FREE;
        this.addParameter(p);

        p = new ParamDefinition(ComputeNodeType.CondDistri, 'Pr', d);
        p.calculability = ParamCalculability.FREE;
        this.addParameter(p);

        p = new ParamDefinition(ComputeNodeType.SectionParametree, 'Pr', d);
        p.calculability = ParamCalculability.FREE;
        this.addParameter(p);

        p = new ParamDefinition(ComputeNodeType.RegimeUniforme, 'Pr', d);
        p.calculability = ParamCalculability.FREE;
        this.addParameter(p);

        p = new ParamDefinition(ComputeNodeType.CourbeRemous, 'Pr', d);
        p.calculability = ParamCalculability.FREE;
        this.addParameter(p);

        // logObject(this._params);
    }

    private hasParameter(prm: ParamDefinition): boolean {
        for (let p of this._params)
            // if (p.computeNodeType == prm.computeNodeType && (p.symbol == prm.symbol || p.alias == prm.symbol))
            if (p.computeNodeType == prm.computeNodeType && p.symbol == prm.symbol)
                return true;

        return false;
    }

    private addParameter(p: ParamDefinition) {
        if (!this.hasParameter(p))
            this._params.push(new NgParameter(p));
    }

    private addParameters(nodeType: ComputeNodeType) {
        let cdp: ParamsEquation = ComputeNodeParameters.getInstance().getComputeNodeParameters(nodeType);
        for (let pi in cdp.map) {
            let p: ParamDefinition = cdp.map[pi];
            this.addParameter(p);
        }
    }

    public getParameter(nodeType: ComputeNodeType, symbol: string): NgParameter {
        for (let p of this._params)
            if (p.computeNodeType == nodeType && p.symbol == symbol)
                return p;

        switch (nodeType) {
            case ComputeNodeType.SectionCercle:
            case ComputeNodeType.SectionPuissance:
            case ComputeNodeType.SectionRectangle:
            case ComputeNodeType.SectionTrapeze:
                return this.getParameter(ComputeNodeType.SectionParametree, symbol);

            case ComputeNodeType.RegimeUniformeCercle:
            case ComputeNodeType.RegimeUniformePuissance:
            case ComputeNodeType.RegimeUniformeRectangle:
            case ComputeNodeType.RegimeUniformeTrapeze:
                return this.getParameter(ComputeNodeType.RegimeUniforme, symbol);

            case ComputeNodeType.CourbeRemousCercle:
            case ComputeNodeType.CourbeRemousPuissance:
            case ComputeNodeType.CourbeRemousRectangle:
            case ComputeNodeType.CourbeRemousTrapeze:
                return this.getParameter(ComputeNodeType.CourbeRemous, symbol);

            default:
                return undefined;
        }
    }
}
